package polyglot.ext.polyj.ast;

import polyglot.ext.polyj.types.*;
import polyglot.ext.polyj.util.*;
import polyglot.ext.polyj.visit.*;

import polyglot.ast.*;
import polyglot.types.*;
import polyglot.util.*;
import polyglot.visit.*;

import polyglot.ext.jl.ast.*;

/**
 * A <code>PolyJArrayAccess</code> is identical to an 
 * <code>ArrayAccess</code>, except that accesses to
 * parametric arrays are translated as <code>$AFetch$</code> methods.
 */
public class PolyJArrayAccess_c extends ArrayAccess_c implements PolyJArrayAccess
{
    /**
     * Creates a <code>NewArrayIndexExpression</code>, accessing an element of
     * <code>Expr</code> <code>array</code> at index <code>index</code>.
     */
    public PolyJArrayAccess_c(Position pos, Expr array, Expr index) 
    {
        super(pos, array, index);
    }

    public void translate(CodeWriter w, Translator tr)
    {
        Context c = tr.context();
        TranslationTable tt
            = ((PolyJTypeSystem)c.typeSystem()).translationTable();
        ArrayType type = (ArrayType)array.type();

        if (type.base() instanceof ParamType) {
            ParamType elementType = (ParamType) type.base();
            PolyJTypeSystem ts = (PolyJTypeSystem) tr.typeSystem();
            ClassType scopedClass = c.currentClass();
            ArrayAccessBinding binding = ts.ArrayAccessBinding(elementType,
                scopedClass);
	    PolyJTranslator pjtr = (PolyJTranslator) tr;
            w.write("((" 
	        + PolyJTranslator.adapterInterfaceFullName(scopedClass) 
		+ ")" + pjtr.whereObjExpr() 
	        + ")." + binding.get() + "(");
            w.begin(0);
            printSubExpr(array, w, tr);
            w.write(",");
            w.allowBreak(0);
            index.del().translate(w, tr);
            w.write(")");
            w.end();
        }
        else
        {
            super.translate(w, tr);
        }
    }
}



