package polyglot.ext.polyj.types;

import polyglot.types.*;
import polyglot.util.*;

import polyglot.ext.jl.types.*;

import polyglot.ext.polyj.ast.*;
import polyglot.ext.polyj.visit.*;

import java.util.*;


class WrapDlg_c implements WrapDlg {
    protected final WrappedClassType wrapped;
    protected final PolyJClassType r;
    protected final ClassType oldEnclosingType;
    protected final ClassType newEnclosingType;
    protected final PolyJTypeWrapper tw;
    protected Map methodMap;
    protected Map constructorMap;
    protected transient PolyJTypeSystem ts;
        
    WrapDlg_c(WrappedClassType wrapped, PolyJClassType r, ClassType oldEnclosingType, ClassType newEnclosingType, PolyJTypeWrapper tw) {
        this.wrapped = wrapped;
        this.r = r; 
        this.oldEnclosingType = oldEnclosingType;
        this.newEnclosingType = newEnclosingType;
        this.tw = tw;
        this.methodMap = new HashMap();
        this.constructorMap = new HashMap();
        this.ts = (PolyJTypeSystem) r.typeSystem();
    }

    //
    // Implement WrapDlg - PolyDlg
    //

    public ClassType.Kind kind() {
        return r.kind();
    }
    
    public List methods() {
        return tw.methods(r, r, wrapped, methodMap);
    }
    
    public List constructors() {
        return tw.constructors(r, r, wrapped, constructorMap);
    }
    
    public MethodInstance originalMethod(MethodInstance mi) {
        return (MethodInstance) methodMap.get(mi);
    }
    
    public ConstructorInstance originalConstructor(ConstructorInstance ci) {
        return (ConstructorInstance) constructorMap.get(ci);
    }
    
    public List fields() {
        return tw.fields(r,r,wrapped);
    }

    public Type superType() {
        return tw.superType(r, wrapped);
    }

    public List interfaces() {
        return tw.interfaces(r, wrapped);
    }

    public String fullName() { 
        return PolyJTranslator.fullNameForClass(r); 
    }
    
    public String name() { 
        return PolyJTranslator.nameForClass(r); 
    }
    
    public polyglot.types.Package package_() { 
        return r.package_(); 
    }
    
    public Flags flags() { 
        return r.flags(); 
    }
    
    public boolean isNested() { 
        return r.isNested(); 
    }
    
    public boolean isLocal() { 
        return r.isLocal(); 
    }
    
    public boolean isAnonymous() { 
        return r.isAnonymous(); 
    }
    
    public ClassType outer() {
        return newEnclosingType;
    }

    public List memberClasses() {
        return tw.memberClasses(r,r,wrapped);
    }

    public ClassType memberClassNamed(String name) {
        return tw.memberClassNamed(name, r, r, wrapped);
    }

    public String translate(Resolver c) {
        return r.translate(c);
    }

    //
    // Implement PolyDlg
    //
    
    public List params() {
        return r.params();
    }

    public ParamType paramNamed(String name) {
        return r.paramNamed(name);
    }

    public boolean isPolymorphic() {
        return r.isPolymorphic();
    }

    public void addParam(String paramName) {
        r.addParam(paramName);
    }

    public void addConstraint(ClassConstraint c) throws SemanticException{
        r.addConstraint(c);
    }

    public boolean isSameImpl(TypeObject t) {
        // NJN: added this case
        if (t instanceof PolyJClassType && tw.args().isEmpty()) {
            return ts.equals(t,r);
        }

	if (t instanceof Type)
	{
		Type tt = (Type) t;
		if (tt.isClass() && ts.isInstantiated(tt.toClass())) {
		    return ts.isSameInst(wrapped, tt.toClass());
		}
	}	
        return false;
    }

    public int hashCodeImpl() {
        return r.hashCode();
    }

    public ClassType wrappedClass() {
        return r;
    }

    public List args() {
	    //hackhack.
        if (ts.equals(tw.wrappedType(),wrapped)) {
            return tw.args();
        } else {
            return Collections.EMPTY_LIST;
        }
    }

    public PolyJTypeWrapper substitutor() {
        return tw;
    }
    
}
