package polyglot.ext.polyj.visit;

import polyglot.ast.*;
import polyglot.frontend.*;
import polyglot.types.*;
import polyglot.util.*;
import polyglot.visit.*;

import polyglot.ext.polyj.ast.*;
import polyglot.ext.polyj.types.*;
import polyglot.ext.polyj.util.*;

import java.util.*;

/**
 * Visits the AST to generate the base bindings in the
 * PolyJ translation table that will be necessary in
 * order to generate adapter interfaces and classes.
 *
 * It does not generate all the instantiation
 * bindings, however, since some cannot be generated until
 * all the adapter bindings (i.e., assisted instantiations)
 * are known.
 */
public class BaseBindingsGenerator extends ContextVisitor
{
    PolyJTypeSystem pjts;

    public BaseBindingsGenerator(Job job, PolyJTypeSystem ts, NodeFactory nf) {
        super(job, ts, nf);
        this.pjts = ts;
    }

    public NodeVisitor enterCall(Node n) throws SemanticException {
        if (n instanceof CanonicalTypeNode) {
            CanonicalTypeNode tn = (CanonicalTypeNode) n;
            Type t = tn.type();
            if (pjts.needsAdapterClass(t)) {
                pjts.addInstantiation(t.toClass());
            }
        }

        if (n instanceof PolyJNewArray) {
            PolyJNewArray e = (PolyJNewArray) n;
            ArrayType type = (ArrayType) e.type();
            if (type.ultimateBase() instanceof ParamType) {
                ParamType param = (ParamType) type.ultimateBase();
                pjts.addNewArrayMethod(param, type.dims());
            }
        }

        if (n instanceof PolyJNew) {
            // figure out if we need a getWhere method, or if we
            // can call the constructor directly
            addGetWhere(((PolyJNew)n).type().toClass());
        }

        if (n instanceof PolyJCall) {
            //if calling a static method, figure out if we need
            // a getWhere method or if we can get the where object
            // directly
            PolyJCall call = (PolyJCall) n;
            MethodInstance mi = call.methodInstance();
            if (mi.flags().isStatic() && mi.container().isClass()) {
                addGetWhere(mi.container().toClass());
            }
        }
        
        if (n instanceof PolyJField) {
            // Same as for calls, static fields may need a getWhere method.
            PolyJField field = (PolyJField) n;
            FieldInstance fi = field.fieldInstance();
            if (fi.flags().isStatic() && fi.container().isClass()) {
                addGetWhere(fi.container().toClass());
            }
        }
        
        return this;
    }

    /**
     * Add a GetWhere method if we're inside a polymorphic class
     * and inst needs a Where object.
     */
    protected void addGetWhere(ClassType inst) {
        ClassType ct = context.currentClassScope();
        if (pjts.needsWhereObject(ct) && pjts.needsWhereObject(inst)) {
	    //System.out.println("adding AI: " + ct + ", " + inst);	
            pjts.addAssistedInstantiation((PolyJClassType) ct, inst);
        }
    }
}
